require 'time'
require 'date'

module UU
  module AppLog
    module ApplicationLog
      # Time conversion helper.
      #
      # @example Intended use
      #   class UU::AppLog::SystemLog::Foo
      #     include TimeConversions
      #
      #     def bar(val)
      #       time = parse_time(val)
      #     end
      #   end
      module TimeConversions

        # Parses the given value into an instance of +Time+ representing time in local time.
        # @param [String, Fixnum, Float] val The input value. Either a String in
        #   ISO8601 format or a number representing one of values supported
        #   by +Time.at+ method.
        # @return [Time] Time object corresponding to the input value.
        # @see http://www.ruby-doc.org/core-1.9.3/Time.html#method-c-at Time.at
        def parse_time(val)
          result = nil
          if val
            if val.kind_of?String
              result = Time.parse(val).getlocal
            elsif (val.kind_of?Fixnum) || (val.kind_of?Float)
              result = Time.at(val / 1000.0).getlocal
            end
          end
          result
        end

        # Converts the given value to a String in ISO8601 format.
        # @param [String, DateTime, Time] val The input value.
        # @return [String] String representing time in ISO8601 format.
        def to_iso8601(val)
          result = nil
          if val
            if val.kind_of?String
              result = Time.parse(val).iso8601(FRACTION_DIGITS)
            elsif val.kind_of?Time
              result = val.iso8601(FRACTION_DIGITS)
            elsif val.kind_of?DateTime
              result = val.to_time.iso8601(FRACTION_DIGITS)
            end
          end
          result
        end

      private
        # Number of digits of fraction seconds.
        # @see http://ruby-doc.org/stdlib-1.9.3/libdoc/time/rdoc/Time.html#method-i-xmlschema Time.xmlschema
        FRACTION_DIGITS = 3

      end
    end
  end
end
